@@ -11,7 +11,7 @@
 // General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, see <http://www.gnu.org/licenses/>.
+// along with this program; if not, see <https://www.gnu.org/licenses/>.
 
 #ifndef STREAMED_TYPE_H
 #define STREAMED_TYPE_H
@@ -24,7 +24,7 @@
 namespace Apertium {
 class StreamedType {
 public:
-  std::wstring TheString;
+  UString TheString;
   Optional<LexicalUnit> TheLexicalUnit;
 };
 }
