@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #ifndef _TMXBUILDER_
 #define _TMXBUILDER_
@@ -20,14 +20,15 @@
 #include <apertium/transfer_data.h>
 #include <string>
 #include <cstdio>
+#include <lttoolbox/input_file.h>
 
 using namespace std;
 
 class TMXBuilder
 {
 private:
-  wstring lang1;
-  wstring lang2;
+  UString lang1;
+  UString lang2;
   unsigned int max_edit;
   unsigned int diagonal_width;
   unsigned int window_size;
@@ -37,40 +38,40 @@ private:
   unsigned int low_limit;
   FILE *freference;
 
-  static wstring nextTU(FILE *input);
-  static wstring restOfBlank(FILE *input);
-  static wstring nextBlank(FILE *input);
-  static wstring xmlize(wstring const &str);
-  static bool compatible(FILE *input, FILE *output, bool lazy = false);
-  void generateTMX(FILE *f1, FILE *f2, FILE *output);
-  void outputTU(FILE *f1, FILE *f2, FILE *output);
-  static vector<wstring> reverseList(vector<wstring> const &v);
-  static vector<wstring> sentenceList(FILE *file);
+  static UString nextTU(InputFile& input);
+  static UString restOfBlank(InputFile& input);
+  static UString nextBlank(InputFile& input);
+  static UString xmlize(UString const &str);
+  static bool compatible(InputFile& input, InputFile& output, bool lazy = false);
+  void generateTMX(InputFile& f1, InputFile& f2, UFILE* output);
+  void outputTU(InputFile& f1, InputFile& f2, UFILE* output);
+  static vector<UString> reverseList(vector<UString> const &v);
+  static vector<UString> sentenceList(InputFile& file);
   static int argmin(int nw, int n, int w);
-  static int * levenshteinTable(vector<wstring> &l1, vector<wstring> &l2, 
+  static int * levenshteinTable(vector<UString> &l1, vector<UString> &l2,
 				unsigned int diagonal_width, unsigned int max_edit);
-  void printTU(FILE *output, wstring const &tu1, wstring const &tu2) const;
-  static wstring filter(wstring const &s);
-  static int weight(wstring const &s);  
+  void printTU(UFILE* output, UString const &tu1, UString const &tu2) const;
+  static UString filter(UString const &s);
+  static int weight(UString const &s);
   static void printTable(int *table, unsigned int nrows, unsigned int ncols);
-  static int editDistance(wstring const &s1, wstring const &s2, unsigned int max_edit);
+  static int editDistance(UString const &s1, UString const &s2, unsigned int max_edit);
   static int min3(int i1, int i2, int i3);
   static int min2(int i1, int i2);
-  void printTUCond(FILE *output, wstring const &s1, wstring const &s2, bool secure_zone);
-  static vector<wstring> extractFragment(vector<wstring> const &text, unsigned int base, 
+  void printTUCond(UFILE* output, UString const &s1, UString const &s2, bool secure_zone);
+  static vector<UString> extractFragment(vector<UString> const &text, unsigned int base,
 					 unsigned int width);
 
-  static bool isRemovablePunct(wchar_t const &c);
-  bool similar(wstring const &s1, wstring const &s2);
+  static bool isRemovablePunct(UChar32 const &c);
+  bool similar(UString const &s1, UString const &s2);
 
-  void splitAndMove(FILE *file, string const &filename);
+  void splitAndMove(InputFile& file, string const &filename);
 public:
-  TMXBuilder(wstring const &l1, wstring const &l2);
+  TMXBuilder(UString const &l1, UString const &l2);
   ~TMXBuilder();
   static bool check(string const &file1, string const &file2, bool lazy = false);
-  void generate(string const &file1, string const &file2, 
+  void generate(string const &file1, string const &file2,
                 string const &outfile="");
-                
+
   void setMaxEdit(int me);
   void setDiagonalWidth(int dw);
   void setWindowSize(int ws);
